@@ -5,7 +5,6 @@ from .serializable import Serializable
 from .eq import DeepEqual
 from ..utils.common import trbk
 
-import pdb
 
 import logging
 # create logger
@@ -17,6 +16,9 @@ class EventListener(object):
     """ Generic interface for listeners that will listen to anything
     """
 
+    def __init__(self, **kwds):
+        super().__init__(**kwds)
+
     def targetChanged(self,  *args, **kwargs):
         """ Informs that an event has happened in a target of
         any type.
@@ -154,6 +156,7 @@ class EventSender(object):
         except Exception as e:
             logger.error('listener ' + str(n) +
                          ' got exception: ' + str(e) + ' ' + trbk(e))
+            raise
 
     def getListenerCount(self):
         return len(self._listeners)
@@ -166,15 +169,15 @@ class DatasetEventSender(EventSender):
     def __init__(self, **kwds):
         super(DatasetEventSender, self).__init__(**kwds)
 
-    def addListener(self, listener, cls=DatasetBaseListener):
-        """ Adds a listener to this. """
+    # def addListener(self, listener, cls=DatasetBaseListener):
+    #     """ Adds a listener to this. """
 
-        super(DatasetEventSender, self).addListener(listener, cls=cls)
+    #     super(DatasetEventSender, self).addListener(listener, cls=cls)
 
-        return self
+    #     return self
 
-    def fire(self, event):
-        super(DatasetEventSender, self).fire(event)
+    # def fire(self, event):
+    #     super(DatasetEventSender, self).fire(event)
 
 
 EventTypes = [
