@@ -14,7 +14,9 @@ class MetaDataHolder(object):
     """
 
     def __init__(self, meta=None, **kwds):
-
+        """ Adds MetaData to the class.
+        with defaults set to self.zInfo['metadata'].
+        """
         if meta is None:
             meta = metadata.MetaData()
         self.setMeta(meta)
