@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-from .productpool import ProductPool
+from .productpool import ManagedPool
 from ..utils.common import pathjoin, trbk
 
 import filelock
@@ -26,28 +26,27 @@ else:
     from urlparse import urlparse, quote, unquote
 
 
-def wipeLocal(poolpath):
+def wipeLocal(path):
     """
     does the scheme-specific remove-all
     """
     # logger.debug()
-    pp = poolpath
-    if pp == '/':
+
+    if path == '/':
         raise(ValueError('Do not remove root directory.'))
 
-    if not op.exists(pp):
+    if not op.exists(path):
         return
     try:
-        shutil.rmtree(pp)
-        os.mkdir(pp)
+        shutil.rmtree(path)
+        os.mkdir(path)
     except Exception as e:
-        msg = 'remove-mkdir ' + pp + \
-            ' failed. ' + str(e) + trbk(e)
+        msg = 'remove-mkdir failed. exc: %s trbk: %s.' % (str(e), trbk(e))
         logger.error(msg)
         raise e
 
 
-class LocalPool(ProductPool):
+class LocalPool(ManagedPool):
     """ the pool will save all products in local computer.
     """
 
@@ -73,7 +72,7 @@ class LocalPool(ProductPool):
         self._atimes = {}
         self._cached_files = {}
 
-        c, t, u = self.readHK()
+        c, t, u = tuple(self.readHK().values())
 
         logger.debug('created ' + self.__class__.__name__ + ' ' + self._poolname +
                      ' at ' + real_poolpath + ' HK read.')
@@ -82,6 +81,8 @@ class LocalPool(ProductPool):
         self._tags.update(t)
         self._urns.update(u)
 
+        return False
+
     def readmmap(self, filename, close=False, check_time=False):
         fp = op.abspath(filename)
         if check_time:
@@ -98,9 +99,9 @@ class LocalPool(ProductPool):
                 # file_obj.seek(0)
             js = file_obj.read()
         except Exception as e:
-            msg = 'Error in HK reading ' + fp + str(e) + trbk(e)
+            msg = 'Error in HK reading. exc: %s trbk: %s.' % (str(e), trbk(e))
             logging.error(msg)
-            raise Exception(msg)
+            raise NameError(msg)
         if 1:  # close:
             file_obj.close()
             if fp in self._files:
@@ -112,27 +113,26 @@ class LocalPool(ProductPool):
             self._atimes[fp] = sr.st_mtime_ns
         return js
 
-    def readHK(self, hktype=None, serialized=False):
+    def readHK(self, hktype=None, serialize_out=False):
         """
         loads and returns the housekeeping data
 
         hktype: one of 'classes', 'tags', 'urns' to return. default is None to return alldirs
-        serialized: if True return serialized form. Default is false.
+        serialize_out: if True return serialized form. Default is false.
         """
         if hktype is None:
             hks = ['classes', 'tags', 'urns']
         else:
             hks = [hktype]
         fp0 = self.transformpath(self._poolname)
-        # with filelock.FileLock(self.lockpath('w'), timeout=5):
-        # if 1:
+
         hk = {}
         for hkdata in hks:
             fp = op.abspath(pathjoin(fp0, hkdata + '.jsn'))
             if op.exists(fp):
                 js = self.readmmap(fp, check_time=True)
                 if js:
-                    if serialized:
+                    if serialize_out:
                         r = js
                     else:
                         from ..dataset.deserialize import deserialize
@@ -140,10 +140,10 @@ class LocalPool(ProductPool):
                     self._cached_files[fp] = js
                 else:
                     # the file hasnot changed since last time we r/w it.
-                    r = self._cached_files[fp] if serialized else \
+                    r = self._cached_files[fp] if serialize_out else \
                         self.__getattribute__('_' + hkdata)
             else:
-                if serialized:
+                if serialize_out:
                     r = '{"_STID":"ODict"}'
                 else:
                     from ..dataset.odict import ODict
@@ -151,16 +151,20 @@ class LocalPool(ProductPool):
             hk[hkdata] = r
             assert r is not None
         logger.debug('HK read from ' + fp0)
-        return (hk['classes'], hk['tags'], hk['urns']) if hktype is None else hk[hktype]
+        if serialize_out:
+            return '{%s}' % ', '.join(('"%s": %s' % (k, v) for k, v in hk.items())) if hktype is None else hk[hktype]
+        else:
+            return hk if hktype is None else hk[hktype]
 
-    def writeJsonmmap(self, fp, data, close=False, check_time=False, **kwds):
+    def writeJsonmmap(self, fp, data, serialize_in=True, serialize_out=False, close=False, check_time=False, **kwds):
         """ write data in JSON from mmap file at fp.
 
         register the file. Leave file open by default `close`.
         data: to be serialized and saved.
         """
         from ..dataset.serializable import serialize
-        js = serialize(data, **kwds)
+
+        js = serialize(data, **kwds) if serialize_in else data
         fp = op.abspath(fp)
         if 1:  # fp not in self._files or self._files[fp] is None:
             file_obj = open(fp, mode="w+", encoding="utf-8")
@@ -201,7 +205,7 @@ class LocalPool(ProductPool):
                                     check_time=True)
         return l
 
-    def schematicSave(self, resourcetype, index, data, tag=None, **kwds):
+    def doSave(self, resourcetype, index, data, tag=None, serialize_in=True, **kwds):
         """
         does the media-specific saving.
 
@@ -211,18 +215,19 @@ class LocalPool(ProductPool):
         fp = pathjoin(fp0, quote(resourcetype) + '_' + str(index))
         try:
             # t0 = time.time()
-            l = self.writeJsonmmap(fp, data, close=True, **kwds)
+            l = self.writeJsonmmap(
+                fp, data, serialize_in=serialize_in, close=True, **kwds)
             l += self.writeHK(fp0)
             # print('tl %.8f %9d' % (time.time()-t0, l))
             logger.debug('HK written')
         except IOError as e:
-            logger.error('Save ' + fp + 'failed. ' + str(e) + trbk(e))
+            logger.error('Save failed. exc: %s trbk: %s.' % (str(e), trbk(e)))
             raise e  # needed for undoing HK changes
         return l
 
-    def schematicLoadProduct(self, resourcetype, index, serialized=False):
+    def doLoad(self, resourcetype, index, serialize_out=False):
         """
-        does the scheme-specific part of loadProduct.
+        does the action of loading.
 
         """
 
@@ -230,7 +235,7 @@ class LocalPool(ProductPool):
         pp = self.transformpath(self._poolname) + '/' + \
             resourcetype + '_' + indexstr
         js = self.readmmap(pp, close=True)
-        if serialized:
+        if serialize_out:
             r = js
         else:
             from ..dataset.deserialize import deserialize
@@ -238,9 +243,9 @@ class LocalPool(ProductPool):
 
         return r
 
-    def schematicRemove(self, resourcetype, index):
+    def doRemove(self, resourcetype, index):
         """
-        does the scheme-specific part of removal of product from pool.
+        does the action of removal of product from pool.
         """
         fp0 = self.transformpath(self._poolname)
         fp = op.abspath(pathjoin(fp0,  quote(resourcetype) + '_' + str(index)))
@@ -253,12 +258,13 @@ class LocalPool(ProductPool):
             os.unlink(fp)
             self.writeHK(fp0)
         except IOError as e:
-            logger.error('Remove ' + fp + 'failed. ' + str(e) + trbk(e))
+            logger.error('Remove failed. exc: %s trbk: %s.' %
+                         (str(e), trbk(e)))
             raise e  # needed for undoing HK changes
 
-    def schematicWipe(self):
+    def doWipe(self):
         """
-        does the scheme-specific remove-all
+        does the action of remove-all
         """
         for n, f in self._files.items():
             if f:
