@@ -1,11 +1,13 @@
 # -*- coding: utf-8 -*-
-import pdb
+
 from . import productref
 from .poolmanager import PoolManager
-from .productpool import ProductPool
-from .urn import Urn, UrnUtils
+from .productpool import ProductPool, makeLockpath
+from .urn import Urn
 from ..dataset.odict import ODict
-import collections
+
+import filelock
+from weakref import finalize
 
 import logging
 # create logger
@@ -13,9 +15,6 @@ logger = logging.getLogger(__name__)
 # logger.debug('level %d' %  (logger.getEffectiveLevel()))
 
 
-#from .productpool import ProductPool
-
-
 class ProductStorage(object):
     """ Logical store created from a pool or a poolURL.
 
@@ -23,6 +22,7 @@ class ProductStorage(object):
 
         pool: if is a string will be taken as a poolname. if is a pool object will be registered with its name,
         poolurl: is sent to the PoolManager with poolname to get the pool object.
+
     """
 
     def __init__(self, pool=None, poolurl=None, **kwds):
@@ -35,23 +35,53 @@ class ProductStorage(object):
         self._pools = ODict()  # dict of poolname - poolobj pairs
         self.register(pool=pool, poolurl=poolurl, **kwds)
 
-    def register(self, pool=None, **kwds):
+    def register(self,  poolname=None, poolurl=None, pool=None, **kwds):
         """ Registers the given pools to the storage.
+
+
         """
 
-        if issubclass(pool.__class__, ProductPool):
-            self._pools[pool.getId()] = pool
-        else:
-            # pool can be None
-            _p = PoolManager.getPool(poolname=pool, **kwds)
-            self._pools[_p.getId()] = _p
+        with filelock.FileLock(makeLockpath('ProdStorage', 'w')), \
+                filelock.FileLock(makeLockpath('ProdStorage', 'r')):
+            if pool and issubclass(pool.__class__, ProductPool):
+                _p = PoolManager.getPool(pool=pool, **kwds)
+            elif poolurl is None and poolname is None:
+                # quietly return for no-arg construction case
+                return
+            else:
+                _p = PoolManager.getPool(
+                    poolname=poolname, poolurl=poolurl, **kwds)
+            self._pools[_p._poolname] = _p
 
         logger.debug('registered pool %s -> %s.' %
                      (str(pool), str(self._pools)))
 
+    def unregister(self, pool=None, **kwds):
+        """ Unregisters the given pools to the storage.
+        """
+
+        with filelock.FileLock(makeLockpath('ProdStorage', 'w')):
+            if issubclass(pool.__class__, ProductPool):
+                poolname = pool.getId()
+            else:
+                poolname = pool
+            if PoolManager.isLoaded(poolname):
+                res = PoolManager.remove(poolname)  # TODO i dentify self
+                # do this after del above
+                del self._pools[poolname]
+                logger.debug('unregistered pool %s -> %s.' %
+                             (str(pool), str(self._pools)))
+            else:
+                logger.info('Pool %s is not registered.' % poolname)
+        return
+
+    def unregisterAll(self):
+        PoolManager.removeAll()
+        self._pools.clear()
+
     def load(self, urnortag):
         """ Loads a product with a URN or a list of products with a tag, from the (writeable) pool.
-        It always creates new ProductRefs. 
+        It always creates new ProductRefs.
         returns productref(s).
         urnortag: urn or tag
         """
@@ -89,7 +119,7 @@ class ProductStorage(object):
         poolName: if the named pool is not registered, registers and saves.
         geturnobjs: mh: returns UrnObjs if geturnobjs is True.
         kwds: options passed to json.dump() for localpools.
-        Returns: one or a list of productref with storage info. 
+        Returns: one or a list of productref with storage info.
         """
 
         if poolname is None:
@@ -149,7 +179,7 @@ class ProductStorage(object):
         if poolname not in self._pools:
             msg = 'pool ' + poolname + ' not found'
             logger.error(msg)
-            raise ValueError(msg)
+            raise NameError(msg)
         return self._pools[poolname]
 
     def getWritablePool(self):
@@ -207,14 +237,12 @@ class ProductStorage(object):
         ret = []
         # search all registered pools
         for poolnm, pool in self._pools.items():
-            c, t, u = pool._classes, pool._tags, pool._urns
             ret += pool.select(query, previous)
         return ret
 
-    def __eq__(self, o):
-        """ has the same urn of the writable pool.
-        """
-        return self.getWritablePool() == o.getWritablePool()
+    def __getstate__(self):
+        """ Can be encoded with serializableEncoder """
+        return OrderedDict(writablePool=self.getWritablePool())
 
     def __repr__(self):
-        return self.__class__.__name__ + ' < pool= ' + str(self._pools) + ' >'
+        return self.__class__.__name__ + '( pool= ' + str(self._pools) + ' )'
