@@ -550,7 +550,7 @@ def getProduct_Or_Component(paths, serialize_out=False):
         modname, ptype = mp[0], mp[1]
         cls = Classes.mapping[ptype]
         mod = importlib.import_module(modname)  # TODO
-        mInfo = getattr(mod, 'ProductInfo')
+        mInfo = getattr(mod, 'Model')
         return serialize(mInfo, indent=4), 'Getting API info for %s OK' % paths[1]
     elif lp >= 3:
         return compo_cmds(paths, mInfo, serialize_out=serialize_out)
