@@ -1,64 +1,45 @@
 # -*- coding: utf-8 -*-
 
-import datetime
-import traceback
-import copy
-import sys
-import os
-
-from fdi.dataset.annotatable import Annotatable
-from fdi.dataset.copyable import Copyable
-from fdi.dataset.odict import ODict
+from fdi.utils.leapseconds import utc_to_tai, tai_to_utc, dTAI_UTC_from_utc, _fallback
 from fdi.dataset.eq import deepcmp
-from fdi.dataset.classes import Classes
-from fdi.dataset.deserialize import deserialize
-from fdi.dataset.quantifiable import Quantifiable
-from fdi.dataset.listener import EventSender, DatasetBaseListener, EventTypes, EventType, EventTypeOf
-from fdi.dataset.composite import Composite
-from fdi.dataset.metadata import Parameter, NumericParameter, MetaData, StringParameter, DateParameter, make_jsonable
-from fdi.dataset.datatypes import DataTypes, DataTypeNames
-from fdi.dataset.attributable import Attributable
-from fdi.dataset.abstractcomposite import AbstractComposite
-from fdi.dataset.datawrapper import DataWrapper, DataWrapperMapper
-from fdi.dataset.dataset import ArrayDataset, TableDataset, CompositeDataset, Column
-from fdi.dataset.ndprint import ndprint
+from fdi.dataset.metadata import make_jsonable
+from fdi.dataset.finetime import FineTime
 from fdi.dataset.datatypes import Vector, Quaternion
-from fdi.dataset.finetime import FineTime, FineTime1, utcobj
-from fdi.dataset.history import History
-from fdi.dataset.baseproduct import BaseProduct
-from fdi.dataset.product import Product
+from fdi.dataset.deserialize import Class_Look_Up
+from fdi.dataset.testproducts import get_sample_product
 from fdi.pal.urn import Urn
 from fdi.utils.checkjson import checkjson
 from fdi.utils.loadfiles import loadcsv
 from fdi.utils import moduleloader
-from fdi.utils.common import fullname, l2t
+from fdi.utils.common import fullname
 from fdi.utils.options import opt
 from fdi.utils.fetch import fetch
 
-# import __builtins__
-
+import traceback
+import copy
+from datetime import timezone, timedelta, datetime
+import sys
+import os
+import os.path
+import pytest
 
 if sys.version_info[0] >= 3:  # + 0.1 * sys.version_info[1] >= 3.3:
     PY3 = True
 else:
     PY3 = False
 
-Classes.updateMapping()
-
 if __name__ == '__main__' and __package__ == 'tests':
     # run by python -m tests.test_dataset
-
-    from outputs import nds2, nds3, out_TableDataset, out_CompositeDataset
+    pass
 else:
     # run by pytest
 
     # This is to be able to test w/ or w/o installing the package
     # https://docs.python-guide.org/writing/structure/
-    from .pycontext import fdi
 
-    from .outputs import nds2, nds3, out_TableDataset, out_CompositeDataset
+    from pycontext import fdi
 
-    from .logdict import logdict
+    from logdict import logdict
     import logging
     import logging.config
     # create logger
@@ -82,45 +63,6 @@ def checkgeneral(v):
         assert false
 
 
-def get_sample_product():
-    """
-    """
-    compo = CompositeDataset()
-    # two arraydsets
-    a1 = [768, 4.4, 5.4E3]
-    a2 = 'ev'
-    a3 = 'arraydset 1'
-    a4 = ArrayDataset(data=a1, unit=a2, description=a3)
-    a5, a6, a7 = [[1.09, 289], [3455, 564]
-                  ], 'count', 'background -- arraydset in compo'
-    a8 = ArrayDataset(data=a5, unit=a6, description=a7)
-    a10 = 'calibration_arraydset'
-    compo.set(a10, a8)
-    # a tabledataset
-    ELECTRON_VOLTS = 'eV'
-    SECONDS = 'sec'
-    t = [x * 1.0 for x in range(5)]
-    e = [2 * x + 100 for x in t]
-    x = TableDataset(description="Example table")
-    x["Time"] = Column(data=t, unit=SECONDS)
-    x["Energy"] = Column(data=e, unit=ELECTRON_VOLTS)
-    # set a tabledataset ans an arraydset, with a parameter in metadata
-    a13 = 'energy_table'
-    # metadata to the dataset
-    compo[a13] = x
-    a11 = 'm1'
-    a12 = StringParameter('EX')
-    compo.meta[a11] = a12
-
-    prodx = Product('complex prod')
-    prodx.meta['extra'] = NumericParameter(description='a different param in metadata',
-                                           value=Vector((1.1, 2.2, 3.3)), valid={(1, 22): 'normal', (30, 33): 'fast'}, unit='meter')
-    prodx[a3] = a4
-    prodx['results'] = compo
-
-    return prodx
-
-
 def test_fetch():
 
     # simple nested structure
@@ -181,60 +123,74 @@ def test_fetch():
     v, s = fetch(["description"], p)
     assert v == p.description
     assert s == '.description'
-    # metadata
-    e = p.meta['extra']
-    v, s = fetch(["meta", "extra"], p)
-    assert v == p.meta['extra']
-    assert s == '.meta["extra"]'
-    # parameter
-    v, s = fetch(["meta", "extra", "unit"], p)
-    assert v == 'meter'
-    assert v == p.meta['extra'].unit
-    assert s == '.meta["extra"].unit'
+    # metadatax
+    with pytest.raises(KeyError):
+        e = p.meta['extra']
 
-    v, s = fetch(["meta", "extra", "value"], p)
+    e = p.meta['speed']
+    v, s = fetch(["meta", "speed"], p)
+    assert v == p.meta['speed']
+    assert s == '.meta["speed"]'
+    # parameter
+    v, s = fetch(["meta", "speed", "unit"], p)
+    assert v == 'meter'
+    assert v == p.meta['speed'].unit
+    assert s == '.meta["speed"].unit'
+
+    v, s = fetch(["meta", "speed", "value"], p)
     assert v == Vector((1.1, 2.2, 3.3))
-    assert v == p.meta['extra'].value
-    assert s == '.meta["extra"].value'
-    v, s = fetch(["meta", "extra", "valid"], p)
+    assert v == p.meta['speed'].value
+    assert s == '.meta["speed"].value'
+    v, s = fetch(["meta", "speed", "valid"], p)
     mkj = make_jsonable({(1, 22): 'normal', (30, 33): 'fast'})
     assert v == mkj
-    assert v == make_jsonable(p.meta['extra'].valid)
-    assert s == '.meta["extra"].valid'
+    assert v == make_jsonable(p.meta['speed'].valid)
+    assert s == '.meta["speed"].valid'
     # TODO written is string
     # [[[1, 22], 'normal'], [[30, 33], 'fast']]
-    v, s = fetch(["meta", "extra", "valid", 0, 1], p)
+    v, s = fetch(["meta", "speed", "valid", 0, 1], p)
     assert v == 'normal'
-    assert v == p.meta['extra'].valid[0][1]
-    assert s == '.meta["extra"].valid[0][1]'
+    assert v == p.meta['speed'].valid[0][1]
+    assert s == '.meta["speed"].valid[0][1]'
     #
     # validate execution
-    v, s = fetch(["meta", "extra", "isValid", ], p)
+    v, s = fetch(["meta", "speed", "isValid", ], p)
     assert v == True
-    assert v == p.meta['extra'].isValid()
-    assert s == '.meta["extra"].isValid()'
-    # datasets
-    v, s = fetch(["arraydset 1", "unit"], p)
-    assert v == 'ev'
-    assert v == p['arraydset 1'].unit
-    assert s == '["arraydset 1"].unit'
+    assert v == p.meta['speed'].isValid()
+    assert s == '.meta["speed"].isValid()'
 
-    v, s = fetch(["arraydset 1", "data"], p)
-    assert v == [768, 4.4, 5.4E3]
-    assert v == p['arraydset 1'].data
-    assert s == '["arraydset 1"].data'
+    # datasets
+    #
+    v, s = fetch(["Temperature", "unit"], p)
+    assert v == 'C'
+    assert v == p['Temperature'].unit
+    assert s == '["Temperature"].unit'
+
+    v, s = fetch(["Temperature", "data"], p)
+    assert v == [768, 767, 766, 4.4, 4.5, 4.6, 5.4E3]
+    assert v == p['Temperature'].data
+    assert s == '["Temperature"].data'
+
+    # dataset has a parameter
+    v, s = fetch(["Temperature", "T0", "tai"], p)
+    assert v == FineTime('2020-02-02T20:20:20.0202').tai
+
+    # a 2D array dataset in compositedataset 'results'
+    v, s = fetch(["results", 'calibration', "unit"], p)
+    assert v == 'count'
+    assert v == p['results']['calibration'].unit
+    assert s == '["results"]["calibration"].unit'
 
     # data of a column in tabledataset within compositedataset
-    v, s = fetch(["results", "energy_table", "Energy", "data"], p)
-    t = [x * 1.0 for x in range(5)]
+    v, s = fetch(["results", "Time_Energy_Pos", "Energy", "data"], p)
+    t = [x * 1.0 for x in range(9)]
     assert v == [2 * x + 100 for x in t]
-    assert v == p['results']['energy_table']['Energy'].data
-    assert s == '["results"]["energy_table"]["Energy"].data'
-    # another dataset in compositedataset 'results'
-    v, s = fetch(["results", 'calibration_arraydset', "unit"], p)
-    assert v == 'count'
-    assert v == p['results']['calibration_arraydset'].unit
-    assert s == '["results"]["calibration_arraydset"].unit'
+    assert v == p['results']['Time_Energy_Pos']['Energy'].data
+    assert s == '["results"]["Time_Energy_Pos"]["Energy"].data'
+    ys, s = fetch(["results", "Time_Energy_Pos", "y"], p)
+    zs, s = fetch(["results", "Time_Energy_Pos", "z"], p)
+    # y^2 + z^2 = 100 for all t
+    assert all((y*y + z*z - 100) < 1e-5 for y, z in zip(ys.data, zs.data))
 
 
 def test_loadcsv():
@@ -354,3 +310,77 @@ def test_opt():
         pass
     else:
         assert 0, 'failed to exit.'
+
+
+def check_conf(cfp, typ, getConfig):
+    cfn = typ + 'local.py'
+    cfp = os.path.expanduser(cfp)
+    filec = os.path.join(cfp, cfn)
+    os.system('rm -f ' + filec)
+    conf = 'import os; %sconfig={"jk":98, "m":os.path.abspath(__file__)}' % typ
+    with open(filec, 'w') as f:
+        f.write(conf)
+    # check conf file directory
+    w = getConfig(conf=typ)
+    assert w['jk'] == 98
+    pfile = w['m']
+    assert pfile.startswith(cfp)
+    os.system('rm -f ' + filec)
+
+
+def test_getConfig_init(getConfig):
+
+    # no arg
+    v = getConfig()
+    from fdi.pns.config import pnsconfig
+    # v is a superset
+    assert all(n in v for n in pnsconfig)
+
+
+def test_getConfig_noENV(getConfig):
+
+    # non-default conf type
+    typ = 'abc'
+    # environment variable not set
+    try:
+        del os.environ['CONF_DIR']
+    except:
+        pass
+
+    # default dir, there is  nothing
+    # put mock in the default directory
+    check_conf('~/.config', typ, getConfig)
+
+
+def test_getConfig_conf(getConfig):
+
+    # non-default conf type
+    typ = 'abc'
+    # specify directory
+    cp = '/tmp'
+    # environment variable
+    os.environ['CONF_DIR'] = cp
+    check_conf(cp, typ, getConfig)
+    # non-existing. the file has been deleted by the check_conf in the last line
+    w = getConfig(conf=typ)
+
+
+def test_leapseconds():
+    t0 = datetime(2019, 2, 19, 1, 2, 3, 456789, tzinfo=timezone.utc)
+    assert dTAI_UTC_from_utc(t0) == timedelta(seconds=37)
+    # the above just means ...
+    assert utc_to_tai(t0) - t0 == timedelta(seconds=37)
+    t1 = datetime(1972, 1, 1, 0, 0, 0, 000000, tzinfo=timezone.utc)
+    assert dTAI_UTC_from_utc(t1) == timedelta(seconds=10)
+    t2 = datetime(1970, 1, 1, 0, 0, 0, 000000, tzinfo=timezone.utc)
+    # interpolation not implemented
+    assert dTAI_UTC_from_utc(t2) == timedelta(seconds=4.213170)
+    t3 = datetime(1968, 2, 1, 0, 0, 0, 000000, tzinfo=timezone.utc)
+    assert dTAI_UTC_from_utc(t2) == timedelta(seconds=4.213170)
+    t1958 = datetime(1958, 1, 1, 0, 0, 0, 0, tzinfo=timezone.utc)
+    assert dTAI_UTC_from_utc(t1958) == timedelta(seconds=0)
+    # leap seconds is added on transition
+    t4 = datetime(2017, 1, 1, 0, 0, 0, 000000, tzinfo=timezone.utc)
+    assert utc_to_tai(t4) - utc_to_tai(t4 - timedelta(seconds=1)) == \
+        timedelta(seconds=2)
+    print(_fallback.cache_info())
